from django.shortcuts import render
from django.db import models
from rest_framework import routers, serializers, viewsets
from django_filters import FilterSet, filters
from django_filters.rest_framework import DjangoFilterBackend

from .models import Pos, Device, Stream


class StreamSerializer(serializers.ModelSerializer):
    class Meta:
        model = Stream
        fields = [ "nx_id", "name" ]


class DeviceSerializer(serializers.ModelSerializer):
    streams = StreamSerializer(many=True)

    class Meta:
        model = Device
        fields = [ "streams" ]


class PosViewSet(viewsets.ModelViewSet):
    queryset = Device.objects.all()
    serializer_class = DeviceSerializer


class PosSerializer(serializers.ModelSerializer):
    device = DeviceSerializer()

    class Meta:
        model = Pos
        fields = [ "text", "received_at", "device" ]


class PosFilter(FilterSet):
    class Meta:
        model = Pos

        fields = {
            "received_at": [ "range" ],
            "text": [ "icontains" ]
        }

    @classmethod
    def filter_for_lookup(cls, f, lookup_type):
        if isinstance(f, models.DateTimeField) and lookup_type=="range":
            return filters.DateTimeFromToRangeFilter, {}

        return super().filter_for_lookup(f, lookup_type)


class PosViewSet(viewsets.ModelViewSet):
    queryset = Pos.objects.order_by("-received_at").all()
    serializer_class = PosSerializer
    filter_class = PosFilter


router = routers.DefaultRouter()
router.register(r"poses", PosViewSet)