import os
import json
from dataclasses import dataclass
from typing import List, Dict

import zmq
from flask import Flask, request, abort, make_response, jsonify, Response
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "PostClient.settings")


### Have to do this for it to work in 1.9.x!
from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()
#############

from PosMgnt.models import Device, Log, Pos, Stream, MediaServer


context = zmq.Context()


def create_app(debug):
    global context
    app = Flask(__name__)

    zmq_url = "tcp://127.0.0.1:6666"
    socket = context.socket(zmq.PUB)
    socket.bind(zmq_url)

    @app.route("/", methods=[ "GET" ])
    def index():
        return make_response(f"Hello: { request.remote_addr }")

    @app.route("/cgi/registry.exe", methods=[ "POST" ])
    def post():
        # Port and Ip are optional
        ip = request.args.get("IP", request.remote_addr)
        port = int(request.args.get("PORT", "1001"))

        # Name is a must
        if "NAME" not in request.args:
            return abort(Response("NAME can't be empty", status=400))

        name = request.args.get("NAME")
        name = name.upper()

        dev: Device = Device.objects.filter(name=name).first()
        if dev:
            dev.status = "online"
            if dev.allowed_to_update:
                if dev.host != ip or dev.port != port:
                    dev.host = ip
                    dev.port = port
            dev.save()
        elif dev is None:
            dev = Device(name=name, host=ip, port=port)
            dev.save()

        socket.send_json({
            "name": dev.name,
            "host": dev.host,
            "port": dev.port
        })

        response = make_response("<html><body><strong>\nSuccessfully Registered!\n</strong></body></html>", 200)
        response.mimetype = "text/html"
        return response


    @app.route("/create_pos", methods=[ "POST" ])
    def create_pos():
        body = request.json
        text: str = body["pos"]
        name: List[int] = body["device"]

        device = Device.objects.filter(name=name).first()
        if device:
            pos = Pos(text=text, device=device)
            pos.save()

            return make_response("OK", 200)
        else:
            return make_response("Not found", 404)


    @app.route("/device/<name>", methods=[ "GET" ])
    def device(name: str):
        dev: Device = Device.objects.filter(name=name).first()
        server_ids = set()

        def format_server(s: MediaServer):
            server_ids.add(s.id)

            return {
                "host": f"{ s.host }:{ s.port }",
                "user": s.username,
                "password": s.password
            }

        def format_dev(dev: Device):
            streams: List[Stream] = dev.streams.all()
            stream_ids = [ s.nx_id for s in streams ]
            servers = [ s.server for s in streams ]

            return {
                "name": dev.name,
                "host": dev.host,
                "port": dev.port,
                "stream_ids": stream_ids,
                "servers": [ format_server(s) for s in servers if s.id not in server_ids ]
            }

        response = make_response(format_dev(dev), 200)
        response.content_type = "application/json"
        response.mimetype = "application/json"
        return response


    @app.route("/offline/<name>", methods=[ "POST" ])
    def offline(name: str):
        dev: Device = Device.objects.filter(name=name).first()
        if dev:
            dev.status = "offline"
            dev.save()

        return make_response("Ok", 200)


    @app.route("/devices", methods=[ "GET" ])
    def devices():
        devs: List[Device] = Device.objects.all()

        def format_dev(dev: Device):
            streams: List[Stream] = dev.streams.all()
            stream_ids = [ s.nx_id for s in streams ]
            servers = [ s.server for s in streams ]

            return {
                "name": dev.name,
                "host": dev.host,
                "port": dev.port
            }

        response = make_response(json.dumps([ format_dev(dev) for dev in devs ]), 200)
        response.content_type = "application/json"
        response.mimetype = "application/json"
        return response


    return app


def run_adserver(**kwargs):
    app = create_app(True)
    app.run(host="0.0.0.0", port=80, debug=True, use_reloader=False)


if __name__ == '__main__':
    run_adserver()
