#!/bin/bash

set -e
set -x

pushd "$( dirname "${BASH_SOURCE[0]}" )"
CURRENT_DIR="$PWD"
popd


APP_PREFIX="/opt/pos-service"
DATABASE_PATH="/opt/pos-service-db"
DATABASE_USER="aigsg"

if [ -f "/etc/systemd/system/aigsg-psql.service" ]; then
    sed -i "s#/etc/newsvc/db#$DATABASE_PATH#g" /etc/systemd/system/aigsg-psql.service

    systemctl daemon-reload
    systemctl enable aigsg-psql
fi

if [ ! -d "$DATABASE_PATH" ]; then
    mkdir -p $DATABASE_PATH
    chown -R $DATABASE_USER $DATABASE_PATH
    su $DATABASE_USER -c "TZ=America/Sao_Paulo $APP_PREFIX/bin/initdb -D $DATABASE_PATH"

    systemctl start aigsg-psql

    echo Wait for psql in 10secs
    sleep 10


    echo Initializing database
    su $DATABASE_USER -c "$APP_PREFIX/bin/createdb"

    echo Setting outside access
    sed -i "s/#listen_addresses/listen_addresses/g" $DATABASE_PATH/postgresql.conf
    sed -i "s/localhost/*/g" $DATABASE_PATH/postgresql.conf
    echo "host      all     all     0.0.0.0/0       md5" >> $DATABASE_PATH/pg_hba.conf

    echo Setup database
    su $DATABASE_USER -c "cd $CURRENT_DIR && $APP_PREFIX/bin/psql -f init.sql"
fi

systemctl restart aigsg-psql
